Attribute VB_Name = "LM628"
Option Explicit

'''''''''''''''''''''''''''''''''''''''''''
'''     Hardware adresses and usage     '''

'LM628s Base addresses
Global Const RAxis = &H100     'Rotation(W)
Global Const TAxis = &H104     'Translation(X)

Global Const RELATIVE = True   'Set corresponding bit to 1
Global Const ABSOLUTE = False  'Set corresponding bit to 0
Global Const CNTPERTURN = 720
Global Const CNTPERMM = 1000
Global Const TOTALTRAVEL = 160   'As Charly said
Public Kaccel As Single
'Global Const KACCEL = 200

'Global Ports(1 To 2) As Long

'LM628 command codes
Global Const RESET = 0          'As expected !
Global Const PORT8 = 5          '8 bits output (not used)
Global Const PORT12 = 6         '12 bits output
Global Const DFH = 2            'Define Home
Global Const SIP = 3            'Set Index Position
Global Const LPEI = &H1B        'Interrupt On Error
Global Const LPES = &H1A        'Stop on Error
Global Const SBPA = &H20        'Set Breakpoint Absolute
Global Const SBPR = &H21        'Set Breakpoint Relative
Global Const MSKI = &H1C        'Mask Interrupt
Global Const RSTI = &H1D        'Reset Interrupt
Global Const LFIL = &H1E        'Load Filter
    'Bits allocation in Control Word for LFIL command
    Global Const LDil = 1
    Global Const LDkd = 2
    Global Const LDki = 4
    Global Const LDkp = 8
    
Global Const UDF = 4            'Update Filter

Global Const LTRJ = &H1F        'Load Trajectory
    'Bits allocation in Control Word for LTRJ command
    Global Const POSREL = 1
    Global Const LDPOS = 2
    Global Const VELOREL = 4
    Global Const LDVELO = 8
    Global Const ACCREL = 16
    Global Const LDACC = 32
    Global Const TMOTOFF = 256
    Global Const STOPABRUPT = 512
    Global Const STOPSOFT = 1024
    Global Const VELOMODE = 2048
    Global Const FWD = 4096
    
Global Const STT = 1            'Start Trajectory

Global Const RDSTAT = 0         'Read Satus adress
    'Bits allocation in RDSTAT:
    Global Const Busy = 1
    Global Const CmdError = 2
    Global Const TrajectComplete = 4
    Global Const IndexPulse = 8
    Global Const WrapAround = 16
    Global Const ExessivePos = 32
    Global Const BkPointReached = 64
    Global Const MototOff = 128
    
Global Const RDSIGS = &HC       'Read Signals Register
Global Const RDIP = 9           'Read Index Position
Global Const RDDP = 8           'Read Desired Position
Global Const RDRP = &HA         'Read Real Position
Global Const RDDV = 7           'Read Desired Velocity
Global Const RDRV = &HB         'Read Real Velocity
Global Const RDSUM = &HD        'Read Integration Sum
    
'8255 IO Board
Global Const GIOA = &H300
    'Bit alocation in GIOA
    Global Const BRAKEON = 8        'output 1 to release brake
    Global Const MOTPWRON = 1       'output: Pulse to switch Motors Supply On
    Global Const MOTPWROFF = 2      'output: Pulse to switch Motors Supply Off
    Global Const ENDCYCLE = 4       'output: ?
    
Global Const GIOB = &H301
    'Bit allocation in GIOB
    Global Const PROTECT = 4        'input: Protection Open
    Global Const GENPWR = 1         'input: Main Power On
    Global Const MOTPWR = 2         'input: Motors Powered
    Global Const FCBUTTON = 8       'input: Fin Cycle
    Global Const FOOTSWITCH = 16    'input: FootSwitch
    
Global Const GIOC = &H302
    'Bit allocation in GIOC
    
Global Const GIOCreg = &H303
    'Bit allocation in GIOCreg
    
'Control PP3 on Motor Control Board
Global Const MOTA = &H110
    'Bit allocation in MOTA
    Global Const OneTurn = 1        'Input (Latched ?)
    Global Const OneStep = &H10     'Input (Latched ?)
    Global Const ENDT = &H20        'Input: Carriage End of Travel
    Global Const DRVTOK = &H40      'Input: ?
    Global Const DRVROK = &H4       'Input: ?
    
Global Const MOTB = &H111
    'Bit allocationin MOTB
    Global Const ENMOTR = 1         ' output: Enable Rotation
    Global Const ENMOTT = 2         ' output: Enable Translation
    
Global Const MOTC = &H112
    'Bit allocation in MOTC
    ' UNUSED '

Global Const MOTCreg = &H113
    'Bit allocation in MOTCreg
    ' UNUSED '
    
'''    End of Hardware adresses and usage     '''
'''''''''''''''''''''''''''''''''''''''''''''''''
'' Some variables to emulate peek an poke when debging w/o hardware
Public vGIOA As Byte
Public vGIOB As Byte
Public vGIOC As Byte
Public vMOTA As Byte
Public vMOTB As Byte
Public vMOTC As Byte
Public vMOTCreg As Byte
Public vRaxis As Byte
Public vTaxis As Byte

'Some current values updated by the timer
Public STATR As Long
Public STATT As Long
Public SIGSR As Long
Public SIGST As Long
Public IPR As Long
Public IPT As Long
Public DPR As Long
Public DPT As Long
Public DVR As Long
Public DVT As Long
Public RVR As Long
Public RVT As Long
Public RPR As Long
Public RPT As Long
Public SUMR As Long
Public SUMT As Long

Public IOSwitchesA As Long   'Integer
Public IOSwitchesB As Long
Public IOSwitchesC As Long

Public MotCtrlStat As Long
Public IOSwitchesP As Long   'Previously read values to detect changes
Public MotCtrlStatP As Long

Global Rpm As Long    ' Rotation speed
Global RStoppedAt As Single
Global TStoppedAt As Single
Global Zeroed As Boolean

Public CurrentVG As Integer 'Location of the vertical guide
Public TargetVG As Integer  'from 0 (up) to MAXVG
Public Const MAXVG = 1000   'Steps to move full range

' InterLock  flags
Global SuspendPoll As Boolean
Global Rtoken As Boolean   'Dialog pending when true
Global Ttoken As Boolean
Global AllowsFreeShaft As Boolean


Public Sub SetEndCycleOn()

    Poke GIOA, Peek(GIOA) Or ENDCYCLE
End Sub

Public Sub SetEndCycleOff()

    Poke GIOA, Peek(GIOA) And Not ENDCYCLE
'    Debug.Print "SetEndCycleOff "; Hex(Not ENDCYCLE)
End Sub
Public Function AdjustFormat(X As Long) As Long
'Returns a 30bits signed value as expected by LM628

    AdjustFormat = X    'Except if ...
    If X < &HC0000000 Then
        AdjustFormat = &HC0000000
    End If
    If X > &H3FFFFFFF Then
        AdjustFormat = &H3FFFFFFF
    End If
'    Debug.Print "Adjust", X, Hex(AdjustFormat)
End Function

Public Sub InitIO()

    #If Nohard = 1 Then
        Debug.Print "****************** Public Sub InitIO()"
    #End If

    Poke GIOCreg, &H8A  'Affect pins direction ?
    Poke MOTCreg, &H91 'Affect pins direction ?
    Poke GIOA, BRAKEON  ' + MOTPWRON ' Release brake only, no power applied
    Poke MOTB, 0    'Disable motor drivers
End Sub

Public Sub EnableDriverR()

    Poke MOTB, Peek(MOTB) Or ENMOTR
End Sub

Public Sub EnableDriverT()

    Poke MOTB, Peek(MOTB) Or ENMOTT
    If Programing Then Form1.TOnOff.Visible = True
End Sub

Public Sub DisableDriverR()

    Poke MOTB, Peek(MOTB) And Not ENMOTR
End Sub

Public Sub DisableDriverT()

    Poke MOTB, Peek(MOTB) And Not ENMOTT
    If Programing Then Form1.TOnOff.Visible = False
End Sub

Public Sub PwrMotOff()
    
    Poke GIOA, Peek(GIOA) Or MOTPWROFF
    Wait 100
    Poke GIOA, Peek(GIOA) And Not MOTPWROFF
    Wait 100
End Sub

Public Sub PwrMotOn()
    
    Poke GIOA, Peek(GIOA) Or MOTPWRON
    Wait 100
    Poke GIOA, Peek(GIOA) And Not MOTPWRON
    Wait 100
End Sub

Public Sub ReadLM628s()
'' TODO '' Remove calls to unused registers

    #If Nohard = 1 Then
        Debug.Print "****************** ReadLM628s "
'        MsgBox "", vbOKOnly, "Public Sub ReadLM628s()"
'        Exit Sub
    #End If
    
    STATR = Peek(RAxis)     'read base adress W/O any test
    STATT = Peek(TAxis)
'    SIGSR = ReadData(RAxis, RDSIGS, 2)
'    SIGST = ReadData(TAxis, RDSIGS, 2)
'    IPR = ReadData(RAxis, RDIP, 4)
'    IPT = ReadData(TAxis, RDIP, 4)
    DPR = ReadData(RAxis, RDDP, 4)
    DPT = ReadData(TAxis, RDDP, 4)
    RPR = ReadData(RAxis, RDRP, 4)
    RPT = ReadData(TAxis, RDRP, 4)
    DVR = ReadData(RAxis, RDDV, 4)
    DVT = ReadData(TAxis, RDDV, 4)
    RVR = ReadData(RAxis, RDRV, 2)
    RVT = ReadData(TAxis, RDRV, 2)
'    SUMR = ReadData(RAxis, RDSUM, 2)
'    SUMT = ReadData(TAxis, RDSUM, 2)
End Sub
Public Function ReadData(Port As Long, Cmde As Byte, Bytes As Integer) As Long
' Obtain values from LM628 readable registers.
' Cmde is the command code, Bytes is the length of data to be read.
    
'    #If Nohard = 1 Then
'        MsgBox "Axis: " & Axis & vbCrLf & "Code: " & Str(Code) & vbCrLf & "Bytes: " & Str(Bytes)
'        Exit Function
'    #End If
    
    GetToken Port
    LM628Poke1 Port, Cmde
    ReadData = RcvData(Port, Bytes)
    FreeToken Port
End Function

Public Function RcvData(Port As Long, Bytes As Integer) As Long
Dim B As Byte
Dim I As Integer
Dim J As Integer
Dim X As Double
Dim T(1 To 4) As Integer

    
'    #If Nohard = 1 Then
'        T(1) = &H0
'        T(2) = &H0
'        T(3) = &H0
'        T(4) = &H1
'
'        For I = 1 To Bytes
'            B = T(I)        'Simulate peek a byte
'            X = X * 256     'Accumulate in a double to void sign interpretation
'            X = X + B
'            Debug.Print I, X
'        Next
'        If X >= 2 ^ 31 Then  'Too large for a signed long
'            X = X - 2 ^ 32   'Convert to a signed long
'        End If
'        RcvData = X
'        Debug.Print RcvData, Hex(RcvData)
'        Exit Function
'    #End If
    
'    GetToken Port
    Wbusy Port
    For I = 1 To Bytes / 2
        For J = 1 To 2
            B = Peek(Port + 1)  ' Data address is at port+1
            X = X * 256
            X = X + B
        Next
        Wbusy Port
    Next
    If X >= 2 ^ 31 Then  'Too large for a signed long
        X = X - 2 ^ 32   'Convert to a signed long
    End If
    RcvData = X
#If Nohard Then
    Debug.Print "Read value "; X
#End If

'    FreeToken Port
End Function

Public Sub HReset(Port As Long)
' Reset the LM628 and reload filter parameters.

Dim R As Byte

    #If Nohard = 1 Then
        Debug.Print "****************** HReset "; Hex(Port)
        Exit Sub
    #End If
    
    SuspendPoll = True
    Wait 100    'Alow current poll to finish
    While R <> &HC0         'Or R <> &HC0
        While R <> &HC4     'Or R <> &HC4
            Poke Port, RESET 'Or a software RESET if hardware not available
            Wait 2          'Wait for LM628 terminates internal reset sequence.
            R = Peek(Port) Or &H40     'Read STSAT
        Wend
        LM628Poke1 Port, RSTI
        LM628Poke2 Port, 0  'Disable all interrupts
        Wait 2
        R = Peek(Port) Or &H40
    Wend
    Wait 2
    LM628Poke1 Port, PORT12
    LM628Poke1 Port, RSTI
    LM628Poke2 Port, 0  'Disable all interrupts
    LM628Poke1 Port, MSKI
    LM628Poke2 Port, 0  'Mask all interrupts
    If Port = RAxis Then Service.SetFilterR
    If Port = TAxis Then Service.SetFilterT
    SuspendPoll = False
End Sub

Public Sub LoadFilters(Port As Long, SI As Byte, kp As Long, ki As Long, kd As Long, il As Long)

    #If Nohard = 1 Then
'        MsgBox "Port: " & Str(Port) & vbCrLf & "SI: " & Str(SI) & vbCrLf & "kp: " & Str(kp) & vbCrLf & "kd: " & Str(kd) & vbCrLf & "ki: " & Str(ki) & vbCrLf & "il: " & Str(ki), _
        vbOKOnly, "Public Sub LoadFilters(Port As Long, SI As Byte, kp As Long, kd As Long, ki As Long, il As Long)"
        Debug.Print "LoadFilters "; Port, Hex(SI), Hex(kp), Hex(ki), Hex(kd), Hex(il)
        Exit Sub
    #End If
    
'    GetToken Port
    LM628Poke1 Port, LFIL
    LM628Poke2 Port, SI * 256 + LDil + LDkd + LDki + LDkp 'Load all parameters
    LM628Poke2 Port, kp
    LM628Poke2 Port, ki
    LM628Poke2 Port, kd
    LM628Poke2 Port, il
'    FreeToken Port
End Sub

Public Sub LoadTrajectory(Port As Long, _
                        Dir As Boolean, _
                        Acc As Long, _
                        RelAcc As Boolean, _
                        Velov As Long, _
                        RelVelo As Boolean, _
                        Pos As Long, _
                        RelPos As Boolean, _
                        Velom As Boolean)
Dim ControlWord As Long

    #If Nohard = 1 Then
        Static N As Integer
        N = N + 1
        Debug.Print "LoadTrajectory (" & Format(N) & ") Port :", Port, N
        Debug.Print "Dir", "Acc", "RelAcc", "VeloV", "RelVelo", "Pos", "RelPos", "Velom"
        Debug.Print Dir, Acc, RelAcc, Velov, RelVelo, AdjustFormat(Pos), RelPos, Velom
        Exit Sub
    #End If

    GetToken Port
    LM628Poke1 Port, LTRJ     'Send command code
    ControlWord = LDPOS + LDVELO + LDACC  'Build Control Word
    If Dir Then ControlWord = ControlWord + FWD
    If RelAcc Then ControlWord = ControlWord + ACCREL
    If RelVelo Then ControlWord = ControlWord + VELOREL
    If RelPos Then ControlWord = ControlWord + POSREL
    If Velom Then ControlWord = ControlWord + VELOMODE
'    Debug.Print Hex(ControlWord)
    Service.Spy.Text = LongToSBin(ControlWord)
    LM628Poke2 Port, ControlWord
''''''Theese Parameters have to be formated . . .
    LM628Poke4 Port, Acc
    LM628Poke4 Port, Velov
    LM628Poke4 Port, AdjustFormat(Pos)
    FreeToken Port
End Sub
Public Sub DoRSTI(Port As Long)
    
    LM628Poke1 Port, RSTI
    LM628Poke2 Port, 0
End Sub

Public Function Peek(Port As Long) As Byte

    #If Nohard = 1 Then
'        Debug.Print "peek "; Hex(Port)
        Select Case Port
            Case GIOA
                Peek = vGIOA
            Case GIOB
                Peek = vGIOB
            Case GIOC
                Peek = vGIOC
            Case MOTA
                Peek = vMOTA
            Case MOTB
                Peek = vMOTB
            Case MOTC
                Peek = vMOTC
            Case vMOTCreg
                Peek = vMOTCreg
'            Case RAxis
'                Peek = vRaxis
'            Case TAxis
'                Peek = vTaxis
            Case Else
                Peek = &HFF
            End Select
        Debug.Print "peek "; Hex(Port), Peek
            
        Exit Function
    #End If
    
    Peek = DlPortReadPortUchar(Port)
End Function

Public Sub Poke(Port As Long, V As Byte) 'don't care if busy
' Send a single byte to an IO.

    #If Nohard = 1 Then
        Debug.Print "Poke "; Hex(Port), Hex(V)
        Select Case Port
            Case GIOA
                vGIOA = V
                TestPeekPoke.Text1 = Str(V)
            Case GIOB
                vGIOB = V
                TestPeekPoke.Text2 = Str(V)
            Case GIOC
                vGIOC = V
                TestPeekPoke.Text3 = Str(V)
            Case MOTA
                vMOTA = V
                TestPeekPoke.Text4 = Str(V)
            Case MOTB
                vMOTB = V
                TestPeekPoke.Text4 = Str(V)
            Case MOTC
                vMOTC = V
                TestPeekPoke.Text5 = Str(V)
            Case vMOTCreg
                vMOTCreg = V
                TestPeekPoke.Text6 = Str(V)
            Case RAxis
                vRaxis = V
                TestPeekPoke.Text7 = Str(V)
            Case TAxis
                vTaxis = V
                TestPeekPoke.Text8 = Str(V)
            End Select
        Exit Sub
    #End If
    
    DlPortWritePortUchar Port, V
End Sub

Public Sub LM628Poke1(Port As Long, V As Byte)
' Send a single byte (command) to one LM628 waiting for not busy.
    
    #If Nohard = 1 Then ' Just skip
        Debug.Print "LM628Poke1 "; Hex(Port), Hex(V)
        Exit Sub
    #End If
    #If Nohard = 2 Then ' Show where we are
        MsgBox "Port: " & Hex(Port) & vbCrLf & "V: " & Hex(V), vbOKOnly, "Public Sub LM628Poke1(Port As Long, V As Byte)"
        Exit Sub
    #End If
    
    Wbusy Port
    DlPortWritePortUchar Port, V
End Sub

Public Sub LM628Poke2(Port As Long, V As Long) 'when busy free
' Send a 16 bit data word as two 8 bits, MSB first into LM628.

#If Nohard Then
    Debug.Print "Poke2", Hex(V), Hex(V \ 256), Hex(V And 255)
#End If
    Wbusy Port
    Poke Port + 1, (V \ 256)    'Shift right
    Poke Port + 1, V And 255    'Mask
End Sub

Public Sub LM628Poke4(Port As Long, V As Long)
' Send a 32 bits data word to one LM628.
Dim Y As Long
    
    Y = V \ 65536           'Right shift 16bits but sign is lost !
    If V < 0 Then Y = Y - 1 'Restore it.
    Y = Y And 65535         'And truncate. Funny isn't it
#If Nohard Then
    Debug.Print "Poke4", Hex(V), Hex(Y), Hex(V And 65535)
#End If
    LM628Poke2 Port, Y  'Send 16 msb checking busy
    LM628Poke2 Port, V And 65535
End Sub

Public Sub Wbusy(Port As Long)
'Wait for LM628 not busy

    #If Nohard = 1 Then
        Exit Sub
    #End If
    
    While Peek(Port) And Busy   'Stay here !
    'Should be (Port + RDSTAT) but since RDSTAT is 0 . . .
    Wend
End Sub

Public Sub GetToken(Port As Long)
' Wait for token free then get it

    #If Nohard = 1 Then
        Debug.Print "---- GetToken", Port
'        Exit Sub
    #End If
    
    If Port = RAxis Then
        While Rtoken = True
            DoEvents    'Camp here until token free
        Wend
        Rtoken = True   'mark not free
'        Debug.Print Port & " Token in use"
    End If
    If Port = TAxis Then
        While Ttoken = True
            DoEvents
        Wend
        Ttoken = True
'        Debug.Print Port & " Token in use"
    End If
End Sub

Public Sub FreeToken(Port As Long)
' Release token

#If Nohard Then
    Debug.Print "---- FreeToken", Port
#End If

    If Port = RAxis Then
        Rtoken = False
'        Debug.Print Port & " Token free"
    End If
    If Port = TAxis Then
        Ttoken = False
'        Debug.Print Port & " Token free"
    End If
End Sub

Public Sub MoveSync(mms As Single, Turns As Single)
' Move both axis simultaneously using relative positioning.
' It rolls the specified turns number at Rpm (global) speed
' while simultaneously moving the carriage by the specified number of millimeters.
' Positive turns values mean clockwise rotation (when looking to the shaft)
' Positive mms values mean the carriage moves to the right.
' Returns only after the command in completed and the hardware is
' ready to accept a new order

' With the mecanical constants of the Sipro hardware, we have a pitch of
' 1mm per turn when the rotation velocity is 0.72 times the translation velocity.
' Thus, VeloT = VeloR / 0.72 * pitch (in mms). Pitch being  mms / Turns.

Dim V As Long
    EnableDriverT
    V = RealVeloR / 0.72 * (Abs(mms) / Abs(Turns)) 'Must be positive.
    LoadTrajectory RAxis, False, 1 + RealVeloR / Kaccel, False, RealVeloR, False, -Turns * CNTPERTURN, True, False
    LoadTrajectory TAxis, False, 1 + V / Kaccel, False, V, False, -mms * CNTPERMM, True, False
    DoRSTI RAxis
    DoRSTI TAxis
    LM628Poke1 RAxis, STT
    LM628Poke1 TAxis, STT
    WaitTrajectoriesComplete
'' TODO '' Replace by read RPT when hard available
#If Nohard Then
    SetRef.CarLoc = SetRef.CarLoc + mms  'Show in SetRef
    SetRef.WaitMe
#End If
    DisableDriverT
End Sub

Public Sub WaitTrajectoriesComplete()

#If Nohard Then
    Debug.Print "WaitTrajectoriesComplete"
'    Wait 500
'    Exit Sub
#End If

    While 1
        If ((Peek(RAxis) And TrajectComplete) = TrajectComplete) And ((Peek(TAxis) And TrajectComplete) = TrajectComplete) Then
            Exit Sub      ' Both terminate
        End If
        DoEvents
'            Wait 100
        Sleep 100
    Wend
End Sub

Public Sub WaitRtrajectoryComplete()

#If Nohard Then
    Debug.Print "WaitRtrajectoryComplete"
'    Wait 500
'    Exit Sub
#End If

    While 1
        If ((Peek(RAxis) And TrajectComplete) = TrajectComplete) Then Exit Sub
        DoEvents
'        Wait 100
        Sleep 100
    Wend
End Sub

Public Sub WaitTtrajectoryComplete()

#If Nohard Then
    Debug.Print "WaitTtrajectoryComplete"
'    Wait 500
'    Exit Sub
#End If

    While 1
        If ((Peek(TAxis) And TrajectComplete) = TrajectComplete) Then Exit Sub
        DoEvents
'        Wait 100
        Sleep 100
    Wend
End Sub

Public Sub MoveTRel(mms As Single)
    
    MoveT mms, RELATIVE
End Sub

Public Sub MoveTAbs(mms As Single)
    
    MoveT mms, ABSOLUTE
End Sub

Public Sub MoveT(mms As Single, AR As Boolean)

    EnableDriverT
    LoadTrajectory TAxis, False, 1 + RealVeloT / Kaccel, False, RealVeloT, False, -mms * CNTPERMM, AR, False
    DoRSTI TAxis    'Clear pending status
    LM628Poke1 TAxis, STT
    WaitTtrajectoryComplete
#If Nohard Then
    If AR = ABSOLUTE Then   'To show carriage move in SetRef
        SetRef.CarLoc = mms
    Else
        SetRef.CarLoc = SetRef.CarLoc + mms
    End If
    SetRef.WaitMe
#End If
    DisableDriverT
End Sub

Public Sub MoveR(Turns As Single, AR As Boolean)
    
    LoadTrajectory RAxis, False, 1 + RealVeloR / Kaccel, False, RealVeloR, False, -Turns * CNTPERTURN, AR, False
    DoRSTI RAxis    'Clear pending status
    LM628Poke1 RAxis, STT
    WaitRtrajectoryComplete
End Sub

Public Sub MoveTRef(mms As Single)
' Go to mms from reference

#If Nohard Then
    Debug.Print "MoveTRef "; RefOffset - mms
#End If
    MoveT mms + RefOffset, ABSOLUTE
End Sub
Public Sub GoToZero()
'Move carriage slowly just away from ENDT switch
'' TODO '' Permettre abandon si pb
Dim SaveVeloT As Long

#If Nohard Then
    Debug.Print "GoToZero "
    Zeroed = True
    ParkVguides
    Exit Sub
#End If
    SaveVeloT = RealVeloT
    RealVeloT = 100000      'Slow while searching for zero
    ParkVguides
    If (Peek(MOTA) And ENDT) <> ENDT Then    'We are not already past ENDT
        While (Peek(MOTA) And ENDT) <> ENDT
            MoveTRel -2       ' move to the left in ?? mms steps up to ENDT
            DoEvents
        Wend
    End If
    While (Peek(MOTA) And ENDT) = ENDT  'We are past ENDT
        MoveTRel 0.05   ' move to the right in small steps
        DoEvents
    Wend
    LM628Poke1 TAxis, DFH   'Define home here
    Zeroed = True
    RealVeloT = SaveVeloT
End Sub





